{% load i18n %}
<div class="event-export">
  <a href="{{ request.path }}?format=google" target="_blank">{% trans "Google Calendar" %}</a>
  &nbsp;&nbsp;&nbsp;
  <a href="{{ request.path }}?format=ical">{% trans "Export iCal" %}</a>
</div>
