{% load wagtailadmin_tags i18n %}
<fieldset>
    <legend>{{ self.heading }}</legend>
    <ul class="fields">
        <li>
            <div class="field {{ field|fieldtype }} {{ field|widgettype }} {{ field_classes }}">
                <div class="field-content">
                    <div class="input {{ input_classes }} ">
                        {% block form_field %}
                            {{ field|render_with_errors }}
                        {% endblock %}
                        {# This span only used on rare occasions by certain types of input #}
                        <span></span>
                    </div>

                    {% if field|has_unrendered_errors %}
                        <p class="error-message">
                            {% for error in field.errors %}
                                <span>{{ error|escape }}</span>
                            {% endfor %}
                        </p>
                    {% endif %}
                </div>
            </div>
        </li>
        {% if self.exceptionTZ %}
        <li>
            <b>{% trans "Time Zone" %}: </b>
            {{ self.exceptionTZ }}
        </li>
        {% endif %}
    </ul>
</fieldset>
